function IsInInstance()
    local name, type = GetInstanceInfo();
    if type == "none" then
        return nil;
    end
    return name;
end

function IsInRaid()
    return GetNumRaidMembers() > 0;
end

function IsInParty()
    return GetNumPartyMembers() > 0;
end

function IsInLFGParty()
    return false;
end

local function _TrimColorCodes(text)
    return text:gsub("%|c%x%x%x%x%x%x%x%x", ""):gsub("%|r", "");
end

function TrimColorCodes(text)
    local filtered = "";
    while text ~= "" do
        local p1, p2, p3 = text:match("^(.-)(%|c%x%x%x%x%x%x%x%x%|Hitem:.-%|h%|r)(.*)$");
        if not p1 then
            filtered = filtered .. _TrimColorCodes(text);
            break;
        end
        filtered = filtered .. _TrimColorCodes(p1) .. p2;
        text = p3;
    end
    return filtered;
end

function PrintToChat(text, channel)
    if channel then
        if not (
            (channel == "RAID"  and not IsInRaid()) or
            (channel == "PARTY" and not IsInParty()) or
            (channel == "GUILD" and not IsInGuild())
        ) then
            SendChatMessage(TrimColorCodes(text), channel);
        end
    else
        print(text);
    end
end

function BuildMsgMatchTable(tab)
    local n, match;
    for n, match in pairs(tab) do
        match[5] = "^" .. (match[1]:gsub("([%%%?%*%-%+%.%^%$%(%)%[%]%|])", "%%%1")) .. "$";
        match[6], match[7] = {}, 0;
        local arg_num, arg_match = 1;
        for arg_match in match[5]:gmatch("%%%%[%%%d%$]-[sd]") do
            arg_num = 0 + (arg_match:match("(%d+)%%%$") or arg_num);
            table.insert(match[6], match[3][arg_num]);
            arg_num = arg_num + 1;
        end
        match[5] = match[5]:gsub("%%%%[%%%d%$]-[sd]", "(.+)");
        local n2, match2;
        for n2, match2 in pairs(tab) do
            if match2[1]:match(match[5]) then
                match[7] = match[7] + 1;
            end
        end
    end
    table.sort(tab, function(a, b) return a[7] < b[7] end);
end

function ParseMsgMatchTable(tab, msg)
    if not tab[1][5] then
        BuildMsgMatchTable(tab);
    end
    local n, match;
    for n, match in pairs(tab) do
        local p = { msg:match(match[5]) };
        if p[1] then
            local args, pos, arg, value = {};
            for pos, arg in pairs(match[6]) do
                args[arg] = p[pos];
            end
            for arg, value in pairs(match[4]) do
                if value == "$self$" then
                    value = UnitName("player");
                end;
                args[arg] = value;
            end
            return match[2], args;
        end
    end
end

function GetCreateField(tab, ...)
    local p = { ... };
    local ret, n, key;
    for n, key in pairs(p) do
        ret = tab[p[n]];
        if not ret then
            ret = {};
            tab[p[n]] = ret;
        end
        tab = ret;
    end
    return ret;
end
